@@ -4,7 +4,7 @@ use warnings;
 use Carp;
 use Data::Dumper;
 use Scalar::Util 'looks_like_number';
-our $VERSION = '1.0.1';
+our $VERSION = '1.0.2';
 
 my $GlobalRef = {};
 my $THIS;
@@ -101,13 +101,13 @@ use constant {
     DUK_ENUM_SORT_ARRAY_INDICES    => (1 << 4),
     DUK_ENUM_NO_PROXY_BEHAVIOR     => (1 << 5),
 
-    DUK_COMPILE_EVAL               => (1 << 0),
-    DUK_COMPILE_FUNCTION           => (1 << 1),
-    DUK_COMPILE_STRICT             => (1 << 2),
-    DUK_COMPILE_SAFE               => (1 << 3),
-    DUK_COMPILE_NORESULT           => (1 << 4),
-    DUK_COMPILE_NOSOURCE           => (1 << 5),
-    DUK_COMPILE_STRLEN             => (1 << 6),
+    DUK_COMPILE_EVAL               => (1 << 3),
+    DUK_COMPILE_FUNCTION           => (1 << 4),
+    DUK_COMPILE_STRICT             => (1 << 5),
+    DUK_COMPILE_SAFE               => (1 << 6),
+    DUK_COMPILE_NORESULT           => (1 << 7),
+    DUK_COMPILE_NOSOURCE           => (1 << 8),
+    DUK_COMPILE_STRLEN             => (1 << 9),
 
     #Flags for duk_def_prop() and its variants
     DUK_DEFPROP_WRITABLE           => (1 << 0),
@@ -136,7 +136,7 @@ sub new {
     $duk->put_prop_string(-2, "PerlGlobalStash");
     $duk->pop();
 
-    $THIS = bless { sub => 1, duk => $duk, heapptr => 0 }, "JavaScript::Duktape::Object";
+    $THIS = bless { duk => $duk, heapptr => 0 }, "JavaScript::Duktape::Object";
 
     ##global methods
 
@@ -287,7 +287,7 @@ use warnings;
 no warnings 'redefine';
 use Scalar::Util 'looks_like_number';
 use Data::Dumper;
-use Config;
+use Config qw( %Config );
 use JavaScript::Duktape::C::libPath;
 use Carp;
 
@@ -311,6 +311,21 @@ use Inline C => config =>
 use Inline C => JavaScript::Duktape::C::libPath::getPath('duktape_wrap.c');
 use Carp;
 
+use Inline C => q{
+    void poke_buffer(IV to, IV from, IV sz) {
+        memcpy( to, from, sz );
+    }
+};
+my $ptr_format = do {
+    my $ptr_size = $Config{ptrsize};
+    $ptr_size == 4 ?
+        "L" : $ptr_size == 8 ?
+        "Q" :
+        die("Unrecognized pointer size");
+};
+sub peek { unpack 'P'.$_[1], pack $ptr_format, $_[0] }
+sub pv_address { unpack($ptr_format, pack("p", $_[0])) }
+
 sub push_perl {
     my $self = shift;
     my $val = shift;
@@ -361,6 +376,12 @@ sub push_perl {
         elsif ($ref eq 'JavaScript::Duktape::Pointer'){
             $self->push_pointer($$val);
         }
+
+        elsif ($ref eq 'JavaScript::Duktape::Buffer'){
+            my $len = defined $$val ? length($$val) : 0;
+            my $ptr = $self->push_fixed_buffer( $len );
+            poke_buffer( $ptr, pv_address( $$val ), $len );
+        }
     } else {
         if (!defined $val){
             $self->push_undefined();
@@ -403,8 +424,12 @@ sub to_perl {
         $ret = $self->get_number($index);
     }
 
-    elsif ($type == JavaScript::Duktape::DUK_TYPE_OBJECT ||
-            $type == JavaScript::Duktape::DUK_TYPE_BUFFER){
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_BUFFER ) {
+        my $ptr = $self->get_buffer_data($index, my $sz );
+        $ret = peek( $ptr, $sz );
+    }
+
+    elsif ($type == JavaScript::Duktape::DUK_TYPE_OBJECT ) {
 
         if ($self->is_function($index)){
             my $ptr = $self->get_heapptr($index);
@@ -717,15 +742,11 @@ package JavaScript::Duktape::Object; {
         my $self = shift;
         my $duk = $self->{duk};
 
-        #don't free if this coming from sub
-        return if $self->{sub};
-
-        ##TODO: Move these to C
-        my $heapptr = delete $self->{heapptr};
-        return if (!$heapptr);
+        my $refcount = delete $self->{refcount};
+        return if (!$refcount);
         $duk->push_global_stash();
         $duk->get_prop_string(-1, "PerlGlobalStash");
-        $duk->push_number($heapptr);
+        $duk->push_number($refcount);
         $duk->del_prop(-2);
         $duk->pop_2();
     }
@@ -794,9 +815,9 @@ package JavaScript::Duktape::Util; {
                 if (@_){
                     #called with special no arg _
                     shift if (ref $_[0] eq 'NOARGS');
-                    $val = jsFunction($duk, $function_heap, $heapptr, 'call', @_);
+                    $val = jsFunction($method, $duk, $function_heap, $heapptr, 'call', @_);
                 } else {
-                    $val = jsFunction($duk, $function_heap, $heapptr);
+                    $val = jsFunction($method, $duk, $function_heap, $heapptr);
                 }
             } else {
                 $val = $duk->to_perl_object(-1);
@@ -809,6 +830,7 @@ package JavaScript::Duktape::Util; {
     }
 
     sub jsFunction {
+        my $methodname = shift;
         my $duk = shift;
         my $heapptr = shift;
         my $constructor = shift || $heapptr;
@@ -873,33 +895,40 @@ package JavaScript::Duktape::Util; {
         return bless $sub, "JavaScript::Duktape::Function";
     }
 
+    my $REFCOUNT = 0;
     sub jsObject {
         my $options = shift;
 
         my $duk         = $options->{duk};
         my $heapptr     = $options->{heapptr};
         my $constructor = $options->{constructor} || $heapptr;
-        ##TODO: Move these to C
-        # to prevent duktape garbage collecting this object
-        # while it's still used in perl land we need to store
-        # it in perl global stash, this will be freed once the
-        # destroy method called of "JavaScript::Duktape::Object"
+
+        #We may push same heapptr on the global stack more
+        #than once, this results in segmentation fault when
+        #we destroy the object and delete heapptr from the
+        #global stash then trying to use it again
+        #TODO : this is really a poor man solution
+        #for this problem, we use a refcounter to create
+        #a unique id for each heapptr, a better solution
+        #would be making sure same heapptr pushed once and not to
+        #be free unless all gone
+        my $refcount = (++$REFCOUNT) + (rand(3));
+
         $duk->push_global_stash();
         $duk->get_prop_string(-1, "PerlGlobalStash");
-        $duk->push_number($heapptr);
+        $duk->push_number($refcount);
         $duk->push_heapptr($heapptr);
         $duk->put_prop(-3); #PerlGlobalStash[heapptr] = object
         $duk->pop_2();
 
         my $type = $duk->get_type(-1);
 
-        ##if this is a function return sub immediately
-        ##other wise return blessed package
         if ($duk->is_function(-1)){
-            return JavaScript::Duktape::Util::jsFunction($duk, $heapptr, $constructor);
+            return JavaScript::Duktape::Util::jsFunction('anon', $duk, $heapptr, $constructor);
         }
 
         return bless {
+            refcount => $refcount,
             duk => $duk,
             heapptr => $heapptr
         }, "JavaScript::Duktape::Object";
@@ -925,7 +954,6 @@ JavaScript::Duktape - Perl interface to Duktape embeddable javascript engine
 
     #set function to be used from javascript land
     $js->set('write' => sub {
-        my $duk = shift;
         print $_[0], "\n";
     });
 
@@ -1078,7 +1106,6 @@ you can translate duktape api to perl
     #and here is how we can implement it in JavaScript::Duktape
 
     $duk->push_c_function(sub {
-        my $duk = shift;
         my $num1 = $duk->get_int(0);
         my $num2 = $duk->get_int(1);
 
